=head1 NAME

Abr::Conf - configuration parser for Abr

=head1 SCOPE OF INTERFACE CONSUMERS

This interface is only intended for consumption from elsewhere
within the abr software.  When making incompatible changes to
this interface, the only consumer that will be considered is
abr.

=head1 SYNOPSIS

  $conf = Abr::Conf->new(path => "/etc/abr/abr.conf");


=head1 CLASS METHODS

=head2 new(path => $path)

Constructor.  Loads the configuration file located at C<$path>.  Performs
validation.  Throws a L<Abr::E::Invalid> if validation fails.

=head1 OBJECT METHODS

=head2 all_files

Returns a list of L<Abr::File>s.

=head2 file_by_name($name)



=cut

use strict;

package Abr::Conf;

use Abr::E;
use Class::InsideOut qw(id private register);
use YAML;

private path => my %path;
private confdata => my %confdata;

sub new {
    my ($class, %init_args) = @_;

    my $self = register($class);
    my $id = id $self;

    if (!defined($init_args{path})) {
        die "abr internal error: must specify path to new";
    }

    $path{$id} = $init_args{path};

    $confdata{$id} = YAML::LoadFile($path{$id});

    if (ref($confdata{$id}) ne 'ARRAY') {
        my $msg = "top-level data structure in " . $init_args{path}
          . " is not an associative array";
        die Abr::E::Invalid->new($msg);
    }

    for my $el (@{$confdata{$id}}) {
        # path is mandatory as per abr.conf(5)
        exists($el->{path}) or die "missing path";
        # default mechanism is keyboard as per abr.conf(5)
        if (!defined($el->{mechanism})) {
             $el->{mechanism} = "keyboard";
	}
        # It's octal, and default mode is 0400 as per abr.conf(5)
        $el->{mode} = defined($el->{mode})
           ? oct($el->{mode})
           : oct("0400");
    }

    return $self;
}

sub file_by_name {
    my ($self, $file) = @_;
    my $id = id $self;

    my @m = grep { $_->{path} eq $file } @{$confdata{$id}};
    if (@m == 1) {
        return @m;
    }
    elsif (@m == 0) {
        die Abr::E::Invalid->new("don't know about: $file");
    }
    elsif (@m > 1) {
        die Abr::E::Config->new("repeated in config: $file");
    }

    die Abr::E::Logic->new("internal error: non-natural number of matches?");
}

sub all_files {
    my ($self) = @_;
    my $id = id $self;

    return @{$confdata{$id}};
}

1;
